#ifndef LNM_HTTP_LOOP_INTERNAL
#define LNM_HTTP_LOOP_INTERNAL

#include <regex.h>

#include "lnm/http/loop.h"

struct lnm_http_router {
  struct lnm_http_router *exact_children[128];
  struct lnm_http_router *single_segment_child;
  lnm_http_route *routes[lnm_http_method_total];
  bool represents_route;
};

typedef struct lnm_http_route_segment_trie {
  struct lnm_http_route_segment_trie *children[128];
  size_t index;
  bool represents_segment;
} lnm_http_route_segment_trie;

lnm_err lnm_http_route_segment_trie_init(lnm_http_route_segment_trie **out);

void lnm_http_route_segment_trie_free(lnm_http_route_segment_trie *trie);

lnm_err lnm_http_route_key_segment_insert(lnm_http_route *route,
                                          const char *key, size_t key_len,
                                          size_t index);
typedef struct lnm_http_step {
  lnm_http_step_fn fn;
  struct lnm_http_step *next;
  bool blocking;
} lnm_http_step;

/**
 * Initialize a first step.
 *
 * @param out where to store pointer to new `lnm_http_step`
 * @param fn step function associated with the step
 */
lnm_err lnm_http_step_init(lnm_http_step **out);

struct lnm_http_route {
  lnm_http_route_segment_trie *key_segments;
  lnm_http_step *step;
};

lnm_err lnm_http_route_init(lnm_http_route **out);

void lnm_http_route_free(lnm_http_route *route);

/**
 * Initialize a new global context object.
 *
 * @param out where to store pointer to new `lnm_http_loop_gctx`
 */
lnm_err lnm_http_loop_gctx_init(lnm_http_loop_gctx **out, void *c_gctx,
                                lnm_http_ctx_init_fn ctx_init,
                                lnm_http_ctx_reset_fn ctx_reset,
                                lnm_http_ctx_free_fn ctx_free);

/**
 * Initialize a new context.
 *
 * @param out where to store pointer to new object
 * @param gctx global context for the loop
 */
lnm_err lnm_http_loop_ctx_init(lnm_http_loop_ctx **out,
                               lnm_http_loop_gctx *gctx);

void lnm_http_loop_ctx_reset(lnm_http_loop_ctx *ctx);

void lnm_http_loop_ctx_free(lnm_http_loop_ctx *ctx);

void lnm_http_loop_process(lnm_http_conn *conn);

#endif
