#include <stdlib.h>

#include "lnm/common.h"
#include "lnm/http/loop.h"
#include "lnm/http/loop_internal.h"
#include "lnm/loop_internal.h"

lnm_err lnm_http_loop_init(lnm_http_loop **out, void *c_gctx,
                           lnm_http_ctx_init_fn ctx_init,
                           lnm_http_ctx_reset_fn ctx_reset,
                           lnm_http_ctx_free_fn ctx_free) {
  lnm_http_loop *hl = calloc(1, sizeof(lnm_http_loop));

  if (hl == NULL) {
    return lnm_err_failed_alloc;
  }

  LNM_RES2(lnm_http_loop_gctx_init((lnm_http_loop_gctx **)&hl->gctx, c_gctx,
                                   ctx_init, ctx_reset, ctx_free),
           free(hl));

  hl->data_read = lnm_http_loop_process;
  hl->data_write = lnm_http_loop_process;
  hl->ctx_init = (lnm_err(*)(void **, void *))lnm_http_loop_ctx_init;
  hl->ctx_free = (void (*)(void *))lnm_http_loop_ctx_free;
  *out = hl;

  return lnm_err_ok;
}

void lnm_http_loop_router_set(lnm_http_loop *hl, lnm_http_router *router) {
  lnm_http_loop_gctx *gctx = hl->gctx;
  gctx->router = router;
}

lnm_err lnm_http_loop_run(lnm_http_loop *hl, uint16_t port,
                          size_t epoll_threads, size_t worker_threads) {
  LNM_RES(lnm_loop_setup(hl, port));
  return lnm_loop_run_multi(hl, epoll_threads, worker_threads);
}

void lnm_http_loop_set_api_key(lnm_http_loop *hl, const char *api_key) {
  lnm_http_loop_gctx *gctx = hl->gctx;
  gctx->api_key = api_key;
}

void lnm_http_loop_set_server(lnm_http_loop *hl, const char *server) {
  lnm_http_loop_gctx *gctx = hl->gctx;
  gctx->server = server;
}
